/**
 * \ingroup voice
 *
 * \file    qaGobiApiVoice.h
 *
 * \brief   Voice Service API function prototypes
 *
 * Copyright: © 2011 Sierra Wireless, Inc. all rights reserved
 *
 */
#ifndef __GOBI_API_VOICE_H__
#define __GOBI_API_VOICE_H__

#ifdef __cplusplus
extern "C" {
#endif

#define MAXUSSDLENGTH 182

/**
 *  This structure contains USS Information
 *
 *  \param  ussDCS
 *          - 1 - ASCII coding scheme
 *          - 2 - 8-BIT coding scheme
 *          - 3 - UCS2
 *
 *  \param  ussLen
 *          - Range 1 to 182
 *
 *  \param  ussData
 *          - Data encoded as per the DCS
 *
 */
struct USSInfo
{
    BYTE ussDCS;
    BYTE ussLen;
    BYTE ussData[MAXUSSDLENGTH];
};

/**
 *  Initiates a USSD operation.
 *
 *  \param  pInfo[IN]
 *          - USS information
 *          - See \ref USSInfo for more details
 *
 *  \return eQCWWAN_ERR_NONE on success, eQCWWAN_xxx error value otherwise
 *
 *  \sa     See qmerrno.h for eQCWWAN_xxx error values
 *
 *  \note   Technology Supported: UMTS\n
 *          Device Supported: MC83x5\n
 *          Timeout: 5 mins
 *
 */
ULONG OriginateUSSD(
    BYTE  *pInfo );

/**
 *  Responds to a USSD request from the network.
 *
 *  \param  pInfo[IN]
 *          - USS information
 *
 *  \return eQCWWAN_ERR_NONE on success, eQCWWAN_xxx error value otherwise
 *
 *  \sa     See qmerrno.h for eQCWWAN_xxx error values
 *
 *  \note   Technology Supported: UMTS\n
 *          Device Supported: MC83x5\n
 *          Timeout: 5 mins
 *
 */
ULONG AnswerUSSD(
    BYTE  *pInfo );

/**
 *  Cancels an in-progress USSD operation.
 *
 *  \param  None
 *
 *  \return eQCWWAN_ERR_NONE on success, eQCWWAN_xxx error value otherwise
 *
 *  \sa     See qmerrno.h for eQCWWAN_xxx error values
 *
 *  \note   Technology Supported: UMTS\n
 *          Device Supported: MC83x5\n
 *          Timeout: 30 Secs
 *
 */
ULONG CancelUSSD();

#ifdef __cplusplus
} /* extern "C" { */
#endif

#endif /* __GOBI_API_VOICE_H__ */
